#include "housenka.h"

    Barriers::Barriers(int width, int height, int numberOfBarriers, int positionX, int positionY,
                int headPositionX, int headPositionY)
    {
    barriersColor = RGB(0, 0, 0);

		if (numberOfBarriers > (width * height / 10))
            MessageBox(NULL, TEXT("Number of barriers too big"), TEXT("Error"), 0);
        if (numberOfBarriers < 0)
            MessageBox(NULL, TEXT("Number of barriers is less than 0"), TEXT("Error"), 0);
        if (width < 0)
            MessageBox(NULL, TEXT("width is less than 0"), TEXT("Error"), 0);
        if (height < 0)
            MessageBox(NULL, TEXT("height is less than 0"), TEXT("Error"), 0);
        
        this->width = width;
        this->height = height;
        this->numberOfBarriers = numberOfBarriers;
        this->positionX = positionX;
        this->positionY = positionY;
        
        clearBarriersGrid();
        setBarriersGrid(headPositionX, headPositionY);
    }
    
    void Barriers::clearBarriersGrid()
    {
			barriersGrid.resize(width);
        int i, j;
        for (i = 0; i < width; i++)
				{
					barriersGrid[i].resize(height);
            for (j = 0; j < height; j++)
                barriersGrid[i][j] = false;

				}
    }
    
    void Barriers::setBarriersGrid(int headPositionX, int headPositionY)
    {
        int barriersSet = 0;
        int i, j;
        int xHeadDistance, yHeadDistance;
        int counter = 0; // to stop perpetual cycle
        int stop = numberOfBarriers * 10000;
        
        while (barriersSet < numberOfBarriers)
        {
            if ((counter++ >= INT_MAX) || (counter > stop))
		            MessageBox(NULL, TEXT("this number of barriers could not be set"), TEXT("Error"), 0);
            i = rand() % width;
            j = rand() % height;
            
            xHeadDistance = i - headPositionX;
            yHeadDistance = j - headPositionY;
            if (xHeadDistance < 0)
                xHeadDistance = -1 * xHeadDistance;
            if (yHeadDistance < 0)
                yHeadDistance = -1 * yHeadDistance;
            
            if ((barriersGrid[i][j] == false) && ((xHeadDistance > 3) || (yHeadDistance > 3)))
            {
                barriersGrid[i][j] = true;
                barriersSet++;
            }
        }
    }
    
    /** Tests if there is a barrier at the specific point.
     * @param x X-coordinate (in cells)
     * @param y Y-coordinate (in cells)
     * @return True if there is a barrier, false otherwise.
     */    
    boolean Barriers::IsBarrier(int x, int y)
    {
        return barriersGrid[x][y];
    }
    
    POINT * Barriers::countCellPosition(int cellX, int cellY, POINT * p)
    {
        return Grid::countCellPosition(cellX, cellY, positionX, positionY, p);
    }
    
    /** Paints barriers in a form.
     */    
    void Barriers::paint(HWND hwnd)
    {
        HBRUSH hBrush;
				RECT rect;
				HDC hdc;
				POINT p;
			
				hBrush = CreateSolidBrush(barriersColor);

				hdc = GetDC(hwnd);
        int i, j;
        for (i = 0; i < width; i++)
            for (j = 0; j < height; j++)
            {
                if (barriersGrid[i][j] == true)
                {
                    countCellPosition(i, j, &p);
										SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
                    FillRect(hdc, &rect, hBrush);
                }
            }
				ReleaseDC(hwnd, hdc);
				DeleteObject(hBrush);
    }
    
    /** Clears barriers from a grid on a form. (Repaints the cells by the background
     * color.)
     * @param g Graphics object used to painting.
     * @param backgroundColor Background color.
     */    
		void Barriers::Delete(HWND hwnd, COLORREF backgroundColor)
    {
        HBRUSH hBrush;
				RECT rect;
				HDC hdc;
        POINT p;
			
				hBrush = CreateSolidBrush(barriersColor);
        
				hdc = GetDC(hwnd);
        int i, j;
        for (i = 0; i < width; i++)
            for (j = 0; j < height; j++)
                if (barriersGrid[i][j] == true)
                {
                    countCellPosition(i, j, &p);
										SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
                    FillRect(hdc, &rect, hBrush);
                }
				ReleaseDC(hwnd, hdc);
				DeleteObject(hBrush);
    }