#include "housenka.h"

Caterpillar::Caterpillar(int width, int height, int positionX, int positionY,
		COLORREF backgroundColor, Grid * grid, int numberOfBarriers, HWND hwnd)
	{
		firstMovingRequest = NULL;
		lastMovingRequest = NULL;
		caterpillarLength = 3;
		finalPointX = -1;
		finalPointY = -1;

		this->width = width;
		this->height = height;
		this->positionX = positionX;
		this->positionY = positionY;
		this->backgroundColor = backgroundColor;
		this->grid = grid;
		//caterpillarCells = new CArray(bool[width][height];
		caterpillarCells.resize(width);
		for (int i = 0; i < width; i++)
		{
			caterpillarCells[i].resize(height);
			for (int j = 0; j < height; j++)
				caterpillarCells[i][j] = false;
		}

		currentlyMoving = rand() % 4 + 1;

		head = new CellPoint();
		switch (currentlyMoving)
		{
		case TO_RIGHT: // right
			head->x = 0;
			head->y = rand() % height;
			break;
		case TO_LEFT:
			head->x = width - 1;
			head->y = rand() % height;
			break;
		case TO_DOWN:
			head->x = rand() % width;
			head->y = 0;
			break;
		case TO_UP:
			head->x = rand() % width;
			head->y = height - 1;
			break;
		}
		head->next = NULL;
		head->previous = NULL;
		tail = head;
		caterpillarCurrentLength = 1;

		barriers = new Barriers(grid->GetWidth(), grid->GetHeight(), numberOfBarriers, positionX, positionY, head->x, head->y);
		createFinalPoint(hwnd);
	}

	Caterpillar::~Caterpillar()
	{
		delete barriers;

		CellPoint * cp;

		while (head)
		{
			cp = head;
			head = head->next;
			delete cp;
		}
	}

	/** Repaints whole caterpillar and all barriers.
	* @param g Graphics object that is used to painting.
	*/    
	void Caterpillar::paint(HWND hwnd)
	{
		POINT p;
		HBRUSH brush;
		HDC hdc;
		RECT rect;

		hdc = GetDC(hwnd);

		brush = CreateSolidBrush(finalPointColor);
		countCellPosition(finalPointX, finalPointY, &p);
		SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
		FillRect(hdc, &rect, brush);
		DeleteObject(brush);

		if (head != NULL)
		{
			if (IsCorrectHeadPosition()) // NULL means correct
			{
				brush = CreateSolidBrush(caterpillarHeadColor);
				countCellPosition(head->x, head->y, &p);
				SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
				FillRect(hdc, &rect, brush);
				DeleteObject(brush);
			}

			brush = CreateSolidBrush(caterpillarBodyColor);
			CellPoint * cp = head->next;
			while ((cp != tail) && (cp != NULL))
			{
				countCellPosition(cp->x, cp->y, &p);
				SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
				FillRect(hdc, &rect, brush);
				cp = cp->next;
			}

			if (cp == tail)
			{
				countCellPosition(cp->x, cp->y, &p);
				SetRect(&rect, p.x + 3, p.y + 3, p.x + 3 + Grid::cellSize - 6, p.y + 3 + Grid::cellSize - 6);
				FillRect(hdc, &rect, brush);
			}

			DeleteObject(brush);
		}

		ReleaseDC(hwnd, hdc);

		barriers->paint(hwnd);
	}

	void Caterpillar::paintHead(HWND hwnd)
	{
		HDC hdc;
		HBRUSH brush;
		POINT p;
		RECT rect;

		if (head == NULL)
			return;

		hdc = GetDC(hwnd);

		if (IsCorrectHeadPosition()) // NULL means correct
		{
			brush = CreateSolidBrush(caterpillarHeadColor);
			countCellPosition(head->x, head->y, &p);
			SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
			FillRect(hdc, &rect, brush);
			DeleteObject(brush);
		}
		if (head->next != NULL)
		{
			brush = CreateSolidBrush(caterpillarBodyColor);
			countCellPosition(head->next->x, head->next->y, &p);
			SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
			FillRect(hdc, &rect, brush);
			DeleteObject(brush);
		}

		ReleaseDC(hwnd, hdc);
	}

	void Caterpillar::clearTail(HWND hwnd)
	{
		HDC hdc;
		HBRUSH brush;
		POINT p;
		RECT rect;

		if (tail == NULL)
			return;

		hdc = GetDC(hwnd);

		brush = CreateSolidBrush(backgroundColor);
		countCellPosition(tail->x, tail->y, &p);
		SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
		FillRect(hdc, &rect, brush);
		DeleteObject(brush);

		if ((tail->previous != NULL) && (tail->previous != head))
		{
			brush = CreateSolidBrush(backgroundColor);
			countCellPosition(tail->previous->x, tail->previous->y, &p);
			SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
			FillRect(hdc, &rect, brush);
			DeleteObject(brush);

			brush = CreateSolidBrush(caterpillarBodyColor);
			SetRect(&rect, p.x + 3, p.y + 3, p.x + 3 + Grid::cellSize - 6, p.y + 3 + Grid::cellSize - 6);
			FillRect(hdc, &rect, brush);
			DeleteObject(brush);
		}

		ReleaseDC(hwnd, hdc);
	}

	POINT * Caterpillar::countCellPosition(int cellX, int cellY, POINT * p)
	{
		return Grid::countCellPosition(cellX, cellY, positionX, positionY, p);
	}

	void Caterpillar::ProcessMovingRequest()
	{
		if (firstMovingRequest == NULL)
			return;
		currentlyMoving = firstMovingRequest->whereToMove;

		if (firstMovingRequest == lastMovingRequest)
			lastMovingRequest = NULL;

		firstMovingRequest = firstMovingRequest->next;
	}

	/** Moves caterpillar's body one cell in direction that has been set by force of
	* <CODE>SetMoveDirection(int)</CODE>. If this movement causes that head is on a
	* barrier or not in a grid, head isn't painted.
	* If is not correct head position, <CODE>Move(Graphics)</CODE> does nothing.
	* @param g Graphics object that is used to repaint caterpillar's position
	*/    
	void Caterpillar::Move(HWND hwnd)
	{
		if (IsCorrectHeadPosition())
		{
			ProcessMovingRequest();
			MoveHead(hwnd);
			TestFinalPoint(hwnd);
			MoveTail(hwnd);
		}
	}

	void Caterpillar::MoveHead(HWND hwnd)
	{
		caterpillarCells[head->x][head->y] = true;
		CellPoint * newHead = new CellPoint();
		newHead->next = head;
		head->previous = newHead;
		newHead->previous = NULL;
		switch (currentlyMoving)
		{
		case TO_RIGHT: // right
			newHead->x = head->x + 1;
			newHead->y = head->y;
			break;
		case TO_LEFT:
			newHead->x = head->x - 1;
			newHead->y = head->y;
			break;
		case TO_DOWN:
			newHead->x = head->x;
			newHead->y = head->y + 1;
			break;
		case TO_UP:
			newHead->x = head->x;
			newHead->y = head->y - 1;
			break;
		}
		head = newHead;

		caterpillarCurrentLength++;

		paintHead(hwnd);
	}

	void Caterpillar::MoveTail(HWND hwnd)
	{
		CellPoint * cp;

		while (caterpillarCurrentLength > caterpillarLength)
		{
			clearTail(hwnd);
			caterpillarCells[tail->x][tail->y] = false;
			cp = tail;
			tail = tail->previous;
			tail->next = NULL;
			delete cp;

			caterpillarCurrentLength--;
		}
	}

	void Caterpillar::TestFinalPoint(HWND hwnd)
	{
		if (IsAtFinalPoint())
		{
			caterpillarLength++;
			createFinalPoint(hwnd);
		}
	}

	/** Tests if caterpillar's head is on a barrier or not in a grid (incorrect
	* position) or is somewhere else in a grid (correct position).
	* @return True if head has correct position, false otherwise.
	*/    
	boolean Caterpillar::IsCorrectHeadPosition()
	{
		if (head != NULL)
		{
			if (!IsCorrectPosition(head->x, head->y))
				return false;

			if (caterpillarCells[head->x][head->y] == true)
				// crashed into herself?
				return false;
		}

		return true;
	}

	boolean Caterpillar::IsCorrectPosition(int x, int y)
	{
		if ((x < 0) || (x >= width) || (y < 0) || (y >= height))
			// is out of grid?
			return false;
		if (barriers->IsBarrier(x, y))
			return false;

		return true;
	}

	boolean Caterpillar::IsAtFinalPoint()
	{
		if ((head->x == finalPointX) && (head->y == finalPointY))
			return true;

		return false;
	}

	/** Call this method to remove caterpillar from grid correctly.
	* Deletes caterpillar from a form. Background color passed to constructor is
	* used.
	* @param g Graphics object used to painting.
	*/
	void Caterpillar::Delete(HWND hwnd)
	{
		HBRUSH brush;
		HDC hdc;
		RECT rect;
		POINT p;

		hdc = GetDC(hwnd);
		brush = CreateSolidBrush(backgroundColor);

		int i, j;
		for (i = 0; i < width; i++)
			for (j = 0; j < height; j++)
				if (caterpillarCells[i][j] == true)
				{
					countCellPosition(i, j, &p);
					SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
					FillRect(hdc, &rect, brush);
				}

				if (IsCorrectHeadPosition())
				{
					countCellPosition(head->x, head->y, &p);
					SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
					FillRect(hdc, &rect, brush);
				}          

				barriers->Delete(hwnd, backgroundColor);

				countCellPosition(finalPointX, finalPointY, &p);
				SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
				FillRect(hdc, &rect, brush);

		DeleteObject(brush);
		ReleaseDC(hwnd, hdc);
	}

	/** Sets future moving direction of caterpillar.
	* @param direction Direction. Use constants provided by this class (TO_RIGHT etc.)
	*/    
	void Caterpillar::SetMoveDirection(int direction)
	{
		int lastMoving;
		int newMoving = 0; //invalid value

		if (lastMovingRequest != NULL)
			lastMoving = lastMovingRequest->whereToMove;
		else
			lastMoving = currentlyMoving;

		switch (lastMoving)
		{
		case TO_RIGHT:
		case TO_LEFT:
			if ((direction == TO_UP) || (direction == TO_DOWN))
			{
				newMoving = direction;
			}
			break;
		case TO_DOWN:
		case TO_UP:
			if ((direction == TO_RIGHT) || (direction == TO_LEFT))
			{
				newMoving = direction;
			}
			break;
		}

		if (newMoving == 0)
			return;

		if (lastMovingRequest == NULL)
		{
			firstMovingRequest = new MovingRequest();
			firstMovingRequest->whereToMove = newMoving;
			firstMovingRequest->next = NULL;
			lastMovingRequest = firstMovingRequest;
		}
		else
		{
			lastMovingRequest->next = new MovingRequest();
			lastMovingRequest = lastMovingRequest->next;
			lastMovingRequest->next = NULL;
			lastMovingRequest->whereToMove = newMoving;
		}
	}

	void Caterpillar::createFinalPoint(HWND hwnd)
	{
		int xHeadDistance, yHeadDistance;
		POINT p;
		RECT rect;
		HBRUSH brush;
		HDC hdc;

		do
		{
			finalPointX = rand() % width;
			finalPointY = rand() % height;

			xHeadDistance = finalPointX - head->x;
			yHeadDistance = finalPointY - head->y;
			if (xHeadDistance < 0)
				xHeadDistance = -1 * xHeadDistance;
			if (yHeadDistance < 0)
				yHeadDistance = -1 * yHeadDistance;
		}
		while(((xHeadDistance <= 3) && (yHeadDistance <= 3))
			|| barriers->IsBarrier(finalPointX, finalPointY)
			|| caterpillarCells[finalPointX][finalPointY]
			|| !pointExistPath(finalPointX, finalPointY)
				|| !pointExistTwoPaths(finalPointX, finalPointY));

		hdc = GetDC(hwnd);
		brush = CreateSolidBrush(finalPointColor);
		countCellPosition(finalPointX, finalPointY, &p);
		SetRect(&rect, p.x + 1, p.y + 1, p.x + 1 + Grid::cellSize - 2, p.y + 1 + Grid::cellSize - 2);
		FillRect(hdc, &rect, brush);
		DeleteObject(brush);

		ReleaseDC(hwnd, hdc);
	}

	/** Return true if there is path from head to [x,y]
	*/
	bool Caterpillar::pointExistPath(int x, int y)
	{
		POINT p;

		vector< vector<bool> > reachablePoints;

		reachablePoints.resize(width);
		int i,j;
		for (i = 0; i < width; i++)
		{
			reachablePoints[i].resize(height);
			for (j = 0; j < height; j++)
				reachablePoints[i][j] = false;
		}

		stack<POINT> toProcess;
		reachablePoints[head->x][head->y] = true;

		p.x = head->x;
		p.y = head->y;
		toProcess.push(p);

		int situation;
		int newX = 0, newY = 0; // initialization for compiler satisfaction
		while (!toProcess.empty())
		{
			p = toProcess.top();
			toProcess.pop();

			for (situation = 0; situation < 4; situation++)
			{
				switch (situation)
				{
				case 0: newX = p.x - 1; newY = p.y; break;
				case 1: newX = p.x + 1; newY = p.y; break;
				case 2: newX = p.x; newY = p.y - 1; break;
				case 3: newX = p.x; newY = p.y + 1; break;
				}

				if (IsCorrectPosition(newX, newY) && !reachablePoints[newX][newY])
				{
					reachablePoints[newX][newY] = true;
					p.x = newX;
					p.y = newY;
					toProcess.push(p);
				}
			}

			if (reachablePoints[x][y])
				return true;
		}

		return false;
	}

	bool Caterpillar::pointExistTwoPaths(int x, int y)
	{
		int numOfPaths = 0;
		if (IsCorrectPosition(x - 1, y)) numOfPaths++;
		if (IsCorrectPosition(x + 1, y)) numOfPaths++;
		if (IsCorrectPosition(x, y - 1)) numOfPaths++;
		if (IsCorrectPosition(x, y + 1)) numOfPaths++;

		if (numOfPaths >= 2)
			return true;

		return false;
	}
