#ifndef _HOUSENKA_H_
#define _HOUSENKA_H_

#include <Windows.h>
#include <wingdi.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include <stack>
using namespace std;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow);
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK MyButtonWndProc (HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK MyEditWndProc (HWND, UINT, WPARAM, LPARAM);

class Grid {
private:
	int width;
	int height;

	int oldWidth;
	int oldHeight;
	int oldLeft;
	int oldTop;
	bool eraseNeeded;
	COLORREF gridColor;
	/** Width and height (cells are squares) of interior of a cell. */    
public:
	const static int cellSize = 12;

	Grid(int width, int height);
	void Paint(COLORREF backgroundColor, HWND hwnd, int left, int top);
	int GetWidthPixels();
	int GetHeightPixels();
	void SetWidth(int width);
	void SetHeight(int height);
	int GetWidth();
	int GetHeight();
	static POINT * countCellPosition(int cellX, int cellY, int positionX, int positionY, POINT * p);

private:
	void PaintWithColor(HWND hwnd, int left, int top, COLORREF c, int width, int height);
};

class Barriers {
private:
    int width;
    int height;
    int positionX;
    int positionY;
    int numberOfBarriers;
    vector< vector<bool> > barriersGrid;
    COLORREF barriersColor;
    
public:
    Barriers(int width, int height, int numberOfBarriers, int positionX, int positionY,
                int headPositionX, int headPositionY);
    boolean IsBarrier(int x, int y);
    void paint(HWND hwnd);
		void Delete(HWND hwnd, COLORREF backgroundColor);

private:
    void clearBarriersGrid();
    void setBarriersGrid(int headPositionX, int headPositionY);
    POINT * countCellPosition(int cellX, int cellY, POINT * p);
};

class Caterpillar {

private:
	class CellPoint{
	public:
		int x;
		int y;
		CellPoint * next;
		CellPoint * previous;
	};

	class MovingRequest
	{
	public:
		int whereToMove;
		MovingRequest * next;
	};

	MovingRequest * firstMovingRequest;
	MovingRequest * lastMovingRequest;

	/** Width - number of cells.
	*/
	int width;
	/** Height - number of cells.
	*/
	int height;
	/** X coordinate of grid where caterpillar moves
	*/
	int positionX;
	/** Y coordinate of grid where caterpillar moves
	*/
	int positionY;

	/** true where is body of caterpillar except head
	*/
	vector<vector<bool> > caterpillarCells;
	int caterpillarLength;

	/** Caterpillar can be shorter then <code>caterpillarLength</code> when moving
	* from the border of grid when starting game.
	*/
	int caterpillarCurrentLength;

	const static COLORREF caterpillarBodyColor = RGB(255, 0, 0);
	const static COLORREF caterpillarHeadColor = RGB(0, 0, 255);
	const static COLORREF finalPointColor = RGB(0, 255, 0);
	COLORREF backgroundColor;

	/** Grid where is this caterpillar moving
	*/
	Grid * grid;

	Barriers * barriers;

	int currentlyMoving;
	CellPoint * head;
	CellPoint * tail;
	int finalPointX;
	int finalPointY;

public:
	/** Constant representing caterpillar moving direction from left to right. */    
	const static int TO_RIGHT = 1;
	/** Constant representing caterpillar moving direction from right to left. */    
	const static int TO_LEFT = 2;
	/** Constant representing caterpillar moving direction downwards. */    
	const static int TO_DOWN = 3;
	/** Constant representing caterpillar moving direction upwards. */    
	const static int TO_UP = 4;

	Caterpillar(int width, int height, int positionX, int positionY,
		COLORREF backgroundColor, Grid * grid, int numberOfBarriers, HWND hwnd);
	~Caterpillar();
	void paint(HWND hwnd);
	void Move(HWND hwnd);
	boolean IsCorrectHeadPosition();
	void Delete(HWND hwnd);
	void SetMoveDirection(int direction);

private:
	 void paintHead(HWND hwnd);
	 void clearTail(HWND hwnd);
	 POINT * countCellPosition(int cellX, int cellY, POINT * p);
	 void ProcessMovingRequest();
	 void MoveHead(HWND hwnd);
	 void MoveTail(HWND hwnd);
	 void TestFinalPoint(HWND hwnd);
	 boolean IsCorrectPosition(int x, int y);
	 boolean IsAtFinalPoint();
	 void createFinalPoint(HWND hwnd);
	 bool pointExistPath(int x, int y);
	 bool pointExistTwoPaths(int x, int y);
};


#endif